diff -p -r -U3 -N -X /home/gpc/p/script/gpcdiff.exclude orig/p/doc/en/todo.texi p/doc/en/todo.texi
--- orig/p/doc/en/todo.texi	Fri Nov 30 03:24:52 2001
+++ p/doc/en/todo.texi	Sat Dec  1 00:06:44 2001
@@ -365,6 +365,7 @@ After the release of GPC 2.1, this secti
 contain the bugs fixed after that release.
 
 @itemize @bullet
+@item 20011130: real numbers that are very slightly larger than a power of 10 are output incorrectly with @samp{Write} on some platforms (e.g., IA32) (ian7.pas)
 @item 20011129: @samp{mod} yields a wrong result when the result should be 0 and the right operand is a signed type (fjf572a.pas)
 @item 20011123: some comparisons on packed subranges don't work correctly (couper1.pas)
 @item 20011122: two-unit bug with objects (adam2.pas) <Pine.GSO.4.30.0108011501180.14499-100000@@math>
diff -p -r -U3 -N -X /home/gpc/p/script/gpcdiff.exclude orig/p/rts/numtodec.pas p/rts/numtodec.pas
--- orig/p/rts/numtodec.pas	Tue Jul 10 03:32:15 2001
+++ p/rts/numtodec.pas	Sat Dec  1 00:04:44 2001
@@ -103,7 +105,11 @@ var
   inline function GetDigit: Char;
   var Digit: Integer;
   begin
-    Digit := Trunc (v / DigitVal);
+    { Do to the inexactness of floating point numbers, the digit computed
+      could become 10. Assume 9 here (which will leave the remainder so
+      big that the same will happen to subsequent digits as well) and let
+      the rounding further down fix it. }
+    Digit := Min (9, Trunc (v / DigitVal));
     v := v - Digit * DigitVal;
     DigitVal := DigitVal / 10;
     GetDigit := Succ ('0', Digit)
diff -p -r -U3 -N -X /home/gpc/p/script/gpcdiff.exclude orig/p/test/ian7.pas p/test/ian7.pas
--- orig/p/test/ian7.pas	Thu Jan  1 01:00:00 1970
+++ p/test/ian7.pas	Sat Dec  1 00:05:26 2001
@@ -0,0 +1,29 @@
+program ian7 (input,output);
+
+var
+  s: String (42);
+  i: Integer;
+
+procedure Test;
+var r: Real;
+begin
+  Inc (i);
+  r := 0;
+  ReadStr (s, r);
+  if Abs (r - 0.1) > 1e-6 then
+    begin
+      WriteLn ('failed ', i, ': ', s);
+      Halt
+    end
+end;
+
+begin
+  i := 0;
+  WriteStr (s, 10.0*0.01);            Test;
+  WriteStr (s, 100.0*0.001);          Test;
+  WriteStr (s, 1.0*0.1);              Test;
+  WriteStr (s, 10.0*0.01 : 0 : 10);   Test;
+  WriteStr (s, 100.0*0.001 : 0 : 10); Test;
+  WriteStr (s, 1.0*0.1 : 0 : 10);     Test;
+  WriteLn ('OK')
+end.
